package ee.coding.beauty.web.controller;

import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import ee.coding.beauty.domain.Content;
import ee.coding.beauty.page.PageResult;
import ee.coding.beauty.query.ContentQueryObject;
import ee.coding.beauty.service.IContentService;
import ee.coding.beauty.service.IMetaService;
import ee.coding.beauty.util.AlbumUtil;
import ee.coding.beauty.util.CacheUtil;
import ee.coding.beauty.util.ViewNameUtil;
import ee.coding.beauty.util.TimeUtil;

@Controller
// 表示这个类使用注解的方式配置URL映射
public class AlbumController {

	@Autowired
	private IContentService contentService;

	@Autowired
	private IMetaService metaService;
	
	private String ipdd = AlbumUtil.getIPDD();
	
	@SuppressWarnings("deprecation")
	@RequestMapping("/")
	public ModelAndView main(HttpServletRequest request, ContentQueryObject qo) throws Exception {
		Map<String, ModelAndView> keyMVMap = CacheUtil.getKeyMVMap();
		ModelAndView mv = new ModelAndView();
		mv.addObject("ipdd", ipdd);
		// 拼接key
		String tag = "";
		String referer = request.getHeader("referer");
		if (qo.getKeyword() != null && qo.getKeyword() != "") {
			mv.addObject("keyword", qo.getKeyword());
		} else if (referer != null) {
			referer = URLDecoder.decode(referer);
			if (referer.contains("tag")) {
				tag = referer.substring(referer.lastIndexOf("/") + 1);
				qo.setKeyword(tag);
				mv.addObject("keyword", tag);
			}
		}
		// String device = DeviceUtil.isMoblie(request) ? "true" : "false";
		String key = "main" + ViewNameUtil.isMoblie(request) + tag + qo.toString();
		// 如果缓存中有，直接返回
		if (keyMVMap.containsKey(key)) {
			return keyMVMap.get(key);
		} else {
			// 缓存中没有，查询后添加到缓存
			PageResult result = contentService.pageQuery(qo);
			mv.addObject("result", result);
			ViewNameUtil.setViewName(request, mv, "index");
			keyMVMap.put(key, mv);
			return mv;
		}
	}

	@RequestMapping("/tag/{tagName}")
	// 根据标签获取套图
	public ModelAndView getByTag(@PathVariable("tagName") String keyword, HttpServletRequest request) throws Exception {
		Map<String, ModelAndView> keyMVMap = CacheUtil.getKeyMVMap();
		String key = "tag" + ViewNameUtil.isMoblie(request) + keyword + TimeUtil.getHourTimeStamp();
		if (keyMVMap.containsKey(key)) {
			return keyMVMap.get(key);
		}
		keyword = new String(keyword.getBytes("iso-8859-1"), "utf-8");
		ModelAndView mv = new ModelAndView();
		mv.addObject("ipdd", ipdd);
		ContentQueryObject qo = new ContentQueryObject();
		qo.setKeyword(keyword);
		PageResult result = contentService.pageQuery(qo);
		mv.addObject("result", result);
		ViewNameUtil.setViewName(request, mv, "index");
		keyMVMap.put(key, mv);
		return mv;
	}

	@RequestMapping("/fanpai/")
	// 点击朕要翻牌时返回随机获取的30套图
	public ModelAndView getRandomAlbum(HttpServletRequest request, ContentQueryObject qo) throws Exception {
		Random random = new Random();
		Set<Content> fanpaiContentSet = CacheUtil.getFanpaiContent(contentService, random);
		ModelAndView mv = new ModelAndView();
		mv.addObject("ipdd", ipdd);
		mv.addObject("content", (Content) fanpaiContentSet.toArray()[random.nextInt(30)]);
		ViewNameUtil.setViewName(request, mv, "random");
		return mv;
	}

	@RequestMapping("/{categorySlug}")
	// 根据分来来获取套图
	public ModelAndView getByCategory(@PathVariable("categorySlug") String categorySlug, HttpServletRequest request,
			ContentQueryObject qo) throws Exception {
		Map<String, ModelAndView> keyMVMap = CacheUtil.getKeyMVMap();
		String key = "category" + ViewNameUtil.isMoblie(request) + categorySlug + qo.toString();
		if (keyMVMap.containsKey(key)) {
			return keyMVMap.get(key);
		}
		ModelAndView mv = new ModelAndView();
		mv.addObject("ipdd", ipdd);
		qo.setCategorySlug(categorySlug);
		PageResult result = contentService.pageQuery(qo);
		mv.addObject("result", result);
		String categoryName = metaService.getNameBySlug(categorySlug);
		mv.addObject("categorySlug", categorySlug);
		mv.addObject("categoryName", categoryName);
		ViewNameUtil.setViewName(request, mv, "index");
		keyMVMap.put(key, mv);
		return mv;
	}

	@RequestMapping("/{categorySlug}/{cid}.html")
	// 根据cid来获取套图，同时获取同标签/分类的6套图
	public ModelAndView getByCid(@PathVariable("cid") Integer cid, HttpServletRequest request) throws Exception {
		Map<String, ModelAndView> keyMVMap = CacheUtil.getKeyMVMap();
		String key = "cid" + ViewNameUtil.isMoblie(request) + cid + TimeUtil.getHourTimeStamp().toString();
		if (keyMVMap.containsKey(key)) {
			return keyMVMap.get(key);
		}
		ModelAndView mv = new ModelAndView();
		mv.addObject("ipdd", ipdd);
		Content content = contentService.get(cid, TimeUtil.getHourTimeStamp());
		if (content == null) {
			ViewNameUtil.setViewName(request, mv, "404page");;
			keyMVMap.put(key, mv);
			return mv;
		}
		mv.addObject("content", content);
		Integer contentCid = content.getCid();
		List<String> tagList = content.getTagList();
		List<Content> contentList = new ArrayList<>();
		for (String tag : tagList) {
			if (contentList.size() == 6) {
				// 如果已经存了6套图，则退出循环
				break;
			}
			ContentQueryObject qo = new ContentQueryObject();
			qo.setKeyword(tag);
			qo.setCurrentPage(1);
			qo.setPageSize(7);
			@SuppressWarnings("unchecked")
			List<Content> data = (List<Content>) contentService.pageQuery(qo).getData();
			for (Content cont : data) {
				List<Content> copyContentList = new ArrayList<>();
				copyContentList.addAll(contentList);
				if (cont.getCid().equals(contentCid)) {
					continue;
				}
				Boolean flag = true;
				for (Content cil : copyContentList) {
					if (cont.getCid().equals(cil.getCid())) {
						flag = false;
						continue;
					}
				}
				if (flag) {
					contentList.add(cont);
					if (contentList.size() == 6) {
						break;
					}
				}
			}
		}
		// 如果根据标签来查询凑不够6套图，则从分类来补
		if (contentList.size() < 6) {
			ContentQueryObject qo = new ContentQueryObject();
			qo.setCategorySlug(content.getCategorySlug());
			qo.setCurrentPage(1);
			qo.setPageSize(7);
			@SuppressWarnings("unchecked")
			List<Content> data = (List<Content>) contentService.pageQuery(qo).getData();
			for (Content cont : data) {
				List<Content> copyContentList = new ArrayList<>();
				copyContentList.addAll(contentList);
				if (cont.getCid().equals(contentCid)) {
					continue;
				}
				Boolean flag = true;
				for (Content cil : copyContentList) {
					if (cont.getCid().equals(cil.getCid())) {
						flag = false;
						continue;
					}
				}
				if (flag) {
					contentList.add(cont);
					if (contentList.size() == 6) {
						break;
					}
				}
			}
		}
		mv.addObject("contentList", contentList);
		mv.addObject("totalCount", contentService.getTotalCount());
		ViewNameUtil.setViewName(request, mv, "album");
		keyMVMap.put(key, mv);
		return mv;
	}

}